#include <iostream>
#include <math.h>
#include "twodvector.h"
using namespace std;
TwoDVector::TwoDVector(double x, double y, bool unit)
{ 
  tdv_x = x; 
  tdv_y = y; 
  unitVector = unit;
  if (unitVector == true) { makeUnit(); }
}

void TwoDVector::makeUnit()
{
  if ((tdv_x == 0) && (tdv_y == 0))
    { cerr << "ERROR, a (0, 0) vector" << endl; return; }
  if (tdv_x == 0)
    { 
      if (tdv_y > 0) { tdv_y = 1.0; return; }
      tdv_y = -1.0; 
      return; 
    }
  if (tdv_y == 0) 
    { 
      if (tdv_x > 0) { tdv_x = 1.0; return; }
      tdv_x = -1.0; 
      return; 
    }
  double magnitude = sqrt(tdv_x * tdv_x + tdv_y * tdv_y);
  tdv_x /= magnitude;
  tdv_y /= magnitude;
}

double TwoDVector::innerProduct(const TwoDVector & v2) const
{
  return (tdv_x * v2.tdv_x + tdv_y * v2.tdv_y);
}

void TwoDVector::reflect(const TwoDVector & vin, TwoDVector & vout) const
{
  if ((unitVector == false) || 
      (vin.unitVector == false) ||
      (vout.unitVector == false))
    { cout << "ERROR! vector must unit" << endl; }
  double s = -2 * innerProduct(vin);
  vout.tdv_x = vin.tdv_x + s * tdv_x;
  vout.tdv_y = vin.tdv_y + s * tdv_y;
  vout.makeUnit();
}

void TwoDVector::print()
{
  cout << "(" << tdv_x << "," << tdv_y << ")" << endl;
}

void TwoDVector::addVector(const TwoDVector & vec)
{
  tdv_x += vec.tdv_x;
  tdv_y += vec.tdv_y;
  if (unitVector == true) { makeUnit(); }
}

void testVectorReflect()
{
  cout << "test 1" << endl;
  TwoDVector vin1(-1, -1);
  TwoDVector nor1(0, 1);
  TwoDVector vout1(0, 1);
  nor1.reflect(vin1, vout1);
  vin1.print();
  nor1.print();
  vout1.print();
  
  cout << endl << "test 2" << endl;
  TwoDVector vin2(-1, 0);
  TwoDVector nor2(1, 1);
  TwoDVector vout2(0, 1);
  nor2.reflect(vin2, vout2);
  vin2.print();
  nor2.print();
  vout2.print();

  cout << endl << "test 3" << endl;
  TwoDVector vin3(-1, -sqrt(3));
  TwoDVector nor3(0, 1);
  TwoDVector vout3(0, 1);
  nor3.reflect(vin3, vout3);
  vin3.print();
  nor3.print();
  vout3.print();

  cout << endl << "test 4" << endl;
  TwoDVector vin4(sqrt(3), 1);
  TwoDVector nor4(-1, 1);
  TwoDVector vout4(0, 1);
  nor4.reflect(vin4, vout4);
  vin4.print();
  nor4.print();
  vout4.print();


}
