#include "userinterface.h"
#include "playfield.h"
#include "constant.h"
#include <iostream>
using namespace std;
UserInterface::UserInterface(int argc, char * argv[])
{
  createControl();
  createStatus();
  ui_centralWidget = new QWidget;
  QVBoxLayout * mainLayout = new QVBoxLayout;
  ui_playField = new PlayField;
  ui_centralWidget -> setLayout(mainLayout); 
  mainLayout -> addWidget(ui_playField);
  mainLayout -> addWidget(ui_controlGroup);
  mainLayout -> addWidget(ui_statusGroup);
  setCentralWidget(ui_centralWidget);
  setWindowTitle(tr("Two-Player Breakout Game"));
  ui_updateTimer = new QTimer;
  ui_updateTimer -> setInterval(GC_updateDelay);
  ui_updateTimer -> setSingleShot(false); // repeat the timer
  connect(ui_updateTimer, SIGNAL(timeout()), this, SLOT(updatePlayField()));
}
UserInterface::~UserInterface()
{
}

void UserInterface::createControl()
{
  QVBoxLayout * controlLayout = new QVBoxLayout;
  QWidget *startExitWidget = new QWidget;
  QHBoxLayout * startExitLayout = new QHBoxLayout;
  ui_startButton = new QPushButton("Start Game");
  QPushButton * exitButton = new QPushButton("Exit");
  startExitLayout -> addWidget(ui_startButton);
  startExitLayout -> addWidget(exitButton);
  startExitWidget -> setLayout(startExitLayout);
  connect(ui_startButton, SIGNAL(clicked()), this, SLOT(startGame()));
  connect(exitButton, SIGNAL(clicked()), this, SLOT(exitGame()));
  controlLayout -> addWidget(startExitWidget);
  ui_controlGroup = new QGroupBox(tr("Control"));
  ui_controlGroup -> setLayout(controlLayout);
}

void UserInterface::createStatus()
{
  QGridLayout * statusLayout = new QGridLayout;
  QLabel * position = new QLabel("position: ");
  ui_positionLabel = new QLabel("      ");
  QLabel * direction = new QLabel("direction: ");
  ui_directionLabel = new QLabel("      ");
  QLabel * leftScore = new QLabel("score: ");
  ui_leftScoreLabel = new QLabel("0");
  QLabel * ballLeft = new QLabel("ball left: ");
  ui_ballLeftLabel = new QLabel("4");
  QLabel * rightScore = new QLabel("score: ");
  ui_rightScoreLabel = new QLabel("0");
  statusLayout -> addWidget(position, 0, 0);
  statusLayout -> addWidget(ui_positionLabel, 0, 1);
  statusLayout -> addWidget(direction, 0, 2);
  statusLayout -> addWidget(ui_directionLabel, 0, 3);
  statusLayout -> addWidget(ballLeft, 0, 4);
  statusLayout -> addWidget(ui_ballLeftLabel, 0, 5);
  statusLayout -> addWidget(leftScore, 1, 0);
  statusLayout -> addWidget(ui_leftScoreLabel, 1, 1);
  statusLayout -> addWidget(rightScore, 1, 4);
  statusLayout -> addWidget(ui_rightScoreLabel, 1, 5);
  ui_statusGroup = new QGroupBox(tr("Status"));
  ui_statusGroup -> setLayout(statusLayout);
}

void UserInterface::startGame()
{
  ui_updateTimer -> start();
}

void UserInterface::exitGame()
{
  // optional: ask the user whether the game should be terminated,
  // allow the user to cancel
  close();
}

void UserInterface::updatePlayField()
{
  ui_playField -> update();
}
